/*
    (C) 1995-97 AROS - The Amiga Research OS
    $Id: requestchoice.c,v 1.4 2012/05/11 23:56:29 jacadcaps Exp $
 
    Desc: RequestChoice CLI command
    Lang: english
*/

/*****************************************************************************
 
    NAME
 
        RequestChoice
 
    SYNOPSIS
 
        TITLE/A,BODY/A,GADGETS/A/M,PUBSCREEN/K,TYPE,TIMEOUT=TIMEOUTSECS/K/N
 
    LOCATION
 
        Workbench:c
 
    FUNCTION
 
        Allows AmigaDOS scripts to have access to the EasyRequest() function
        for input.
 
    INPUTS
 
        TITLE       - The text to display in the title bar of the requester.
 
        BODY        - The text to display in the body of the requester.
 
        GADGETS     - The text for each of the buttons.
 
        PUBSCREEN   - The name of the public screen to open the requester
                      upon.

        TYPE        - Type of the requester. INFORMATION, QUESTION, CRASH etc.
                      Unknown types will be quietly ignored.

        TIMEOUT=TIMEOUTSECS
                    - Number of seconds to wait before closing the requester
                      and returning the rightmost gadget selected (0).
 
    RESULT
 
        Standard DOS return codes.
 
    NOTES
 
        To place a newline into the body of the requester use *n or *N.
 
        To place a quotation mark in the body of the requester use *".
 
        The CLI template gives the GADGETS option as ALWAYS given; this
        is different from the original program. This way, we do not have
        to check to see if the gadgets have been given.
 
    EXAMPLE
 
        RequestChoice "This is a title" "This is*Na body" Okay|Cancel
 
            This is self-explanitory, except for the "*N". This is the
            equivalent of using a '\n' in C to get a newline in the body
            of the requester. This requester will open on the Workbench
            screen.
 
        RequestChoice Title="This is a title" Body="This is*Na body"
                      Gadgets=Okay|Cancel PubScreen=DOPUS.1
 
            This will do exactly the same as before except that it will
            open on the Directory Opus public screen.
 
    BUGS
 
    SEE ALSO
 
        EasyRequest(), EasyRequestArgs()
 
    INTERNALS
 
    HISTORY

        25-Feb-2006     piru        Added TYPE/K and TIMEOUT=TIMEOUTSECS/K/N
 
        08-Sep-1997     srittau     Use dynamic buffer for gadget-labels
                                    Small changes/fixes
 
        27-Jul-1997     laguest     Initial inclusion into the AROS tree
 
******************************************************************************/

#include <dos/dos.h>
#include <dos/exall.h>
#include <dos/rdargs.h>
#include <exec/libraries.h>
#include <exec/memory.h>
#include <exec/types.h>
#include <intuition/intuition.h>
#include <intuition/screens.h>

#include <string.h>

#include <proto/dos.h>
#include <proto/exec.h>
#include <proto/intuition.h>   /* This causes a spilled register error */

#define ARG_TEMPLATE    "TITLE/A,BODY/A,GADGETS/A/M,PUBSCREEN/K,TYPE/K,TIMEOUT=TIMEOUTSECS/K/N"
enum
{
	ARG_TITLE,
	ARG_BODY,
	ARG_GADGETS,
	ARG_PUBSCREEN,
	ARG_TYPE,
	ARG_TIMEOUT,
	TOTAL_ARGS
};

/* To define whether a command line switch was set or not.
 */
#define NOT_SET         0

#include "requestchoice_version.h"

static const char version[] = VERSTAG;

static const char ERROR_HEADER[] = "RequestChoice";

struct localdata
{
	struct ExecBase		*SysBase;
	struct DosLibrary	*DOSBase;
	struct IntuitionBase	*IntuitionBase;
};
#define SysBase			ld->SysBase
#define DOSBase			ld->DOSBase
#define IntuitionBase	ld->IntuitionBase

static
int Do_RequestChoice(struct localdata *, STRPTR, STRPTR, STRPTR *, STRPTR, STRPTR, LONG *);

int main(void)
{
	struct localdata _ld, *ld = &_ld;
	int             Return_Value;

	Return_Value = RETURN_OK;

	SysBase = *((struct ExecBase**) 4);

	DOSBase = (APTR) OpenLibrary("dos.library",37);
	if (DOSBase)
	{
		IntuitionBase = (APTR) OpenLibrary("intuition.library",37);
		if (IntuitionBase)
		{
			IPTR          * args[TOTAL_ARGS] = { NULL };
			struct RDArgs * rda;

			rda = ReadArgs(ARG_TEMPLATE, (IPTR *)args, NULL);
			if (rda)
			{
				Return_Value = Do_RequestChoice(ld,
				                                (STRPTR)args[ARG_TITLE],
				                                (STRPTR)args[ARG_BODY],
				                                (STRPTR *)args[ARG_GADGETS],
				                                (STRPTR)args[ARG_PUBSCREEN],
				                                (STRPTR)args[ARG_TYPE],
				                                (LONG *)args[ARG_TIMEOUT]);
				FreeArgs(rda);
			}
			else
			{
				PrintFault(IoErr(), ERROR_HEADER);
				Return_Value = RETURN_FAIL;
			}

			CloseLibrary((struct Library *) IntuitionBase);
		}

		CloseLibrary((struct Library *) DOSBase);
	}

	return Return_Value;
}


static
STRPTR ComposeGadgetText(struct localdata *, STRPTR *);

static
int Do_RequestChoice(struct localdata *ld,
                     STRPTR   Title,
                     STRPTR   Body,
                     STRPTR * Gadgets,
                     STRPTR   PubScreen,
                     STRPTR   Type,
                     LONG *   Timeout)
{
	struct Screen     * Scr;
	STRPTR              GadgetText;
	STRPTR              FilteredBodyText = NULL;
	int                 Return_Value;

	Return_Value     = RETURN_OK;

	GadgetText = ComposeGadgetText(ld, Gadgets);
	if (!GadgetText)
		return RETURN_FAIL;

	if (Body)
	{
		STRPTR in = Body;
		ULONG l = 0;
		BOOL filterit = FALSE;
		
		while (*in)
		{
			switch (*in)
			{
			case '%': l++; filterit = TRUE; break;
			}

			l ++;
			in ++;
		}

		if (filterit)
		{
			FilteredBodyText = AllocVec(l + 1, MEMF_PUBLIC);
			
			if (FilteredBodyText)
			{
				STRPTR out = FilteredBodyText;

				in = Body;
				
				while (*in)
				{
					switch (*in)
					{
						case '%': *out = '%'; out ++; *out = '%'; break;
						default: *out = *in;
					}
					
					out ++;
					in ++;
				}
				
				*out = 0;
				
				Body = FilteredBodyText;
			}
			else
			{
				FreeVec(GadgetText);
				return RETURN_FAIL;
			}
		}
	}

	/* Make sure we can open the requester on the specified screen.
	 *
	 * If the PubScreen argument is not specified it will contain
	 * NULL, and hence open on the Workbench screen.
	 */
	Scr = (struct Screen *)LockPubScreen((UBYTE *)PubScreen);
	if (Scr != NULL)
	{
		struct EasyStruct ChoiceES;
		LONG                Result;

		ChoiceES.es_StructSize   = sizeof(ChoiceES);
		ChoiceES.es_Flags        = 0L;
		ChoiceES.es_Title        = Title;
		ChoiceES.es_TextFormat   = Body;
		ChoiceES.es_GadgetFormat = GadgetText;

		/* Open the requester.
		 */
		if (Timeout)
		{
			struct Window *Window;

			Window = BuildEasyRequestArgs(Scr->FirstWindow, &ChoiceES, NULL, NULL);
			UnlockPubScreen(NULL, Scr);

			Result = -1;

			if ((IPTR) Window == 0)
			{
				Result = 0;
			}
			else if ((IPTR) Window == 1)
			{
				Window = NULL;
				Result = 1;
			}

			if (Window)
			{
				struct MsgPort msgport;

				msgport.mp_SigBit = AllocSignal(-1);
				if ((BYTE) msgport.mp_SigBit != -1)
				{
					struct timerequest ioreq;

					msgport.mp_Node.ln_Type = NT_MSGPORT;
					msgport.mp_Flags        = PA_SIGNAL;
					msgport.mp_SigTask      = FindTask(NULL);
					NEWLIST(&msgport.mp_MsgList);

					ioreq.tr_node.io_Message.mn_Node.ln_Type = NT_MESSAGE;
					ioreq.tr_node.io_Message.mn_ReplyPort    = &msgport;
					ioreq.tr_node.io_Message.mn_Length       = sizeof(ioreq);

					if (OpenDevice(TIMERNAME, UNIT_VBLANK, (APTR) &ioreq, 0) == 0)
					{
						ULONG timermask, windowmask;

						ioreq.tr_node.io_Command = TR_ADDREQUEST;
						ioreq.tr_time.tv_secs    = *Timeout;
						ioreq.tr_time.tv_micro   = 0;
						SendIO((APTR) &ioreq);

						windowmask = 1UL << Window->UserPort->mp_SigBit;
						timermask  = 1UL << msgport.mp_SigBit;

						for (;;)
						{
							ULONG sigs;

							sigs = Wait(SIGBREAKF_CTRL_C | windowmask | timermask);

							if (sigs & windowmask)
							{
								Result = SysReqHandler(Window, NULL, FALSE);
								if (Result != -2)
								{
									AbortIO((APTR) &ioreq);
									(void) WaitIO((APTR) &ioreq);

									break;
								}
							}
							if (sigs & (SIGBREAKF_CTRL_C | timermask))
							{
								AbortIO((APTR) &ioreq);
								(void) WaitIO((APTR) &ioreq);
								Result = 0;

								break;
							}
						}

						FreeSysRequest(Window);
						Window = NULL;

						CloseDevice((APTR) &ioreq);
					}

					FreeSignal(msgport.mp_SigBit);
				}

				/* In case something failed, process without timeout */
				if (Window)
				{
					while ((Result = SysReqHandler(Window, NULL, TRUE)) == -2)
					{
					}

					FreeSysRequest(Window);
				}
			}
		}
		else
		{
			Result = EasyRequestArgs(Scr->FirstWindow, &ChoiceES, NULL, NULL);
			UnlockPubScreen(NULL, Scr);
		}

		if (Result != -1)
		{
			Printf("%ld\n", Result);
		}
		else
		{
			Return_Value = RETURN_FAIL;
			SetIoErr(ERROR_NO_FREE_STORE);
			PrintFault(IoErr(), ERROR_HEADER);
		}
	}
	else
	{
		Return_Value = RETURN_FAIL;
		SetIoErr(ERROR_NO_FREE_STORE);
		PrintFault(IoErr(), ERROR_HEADER);
	}

	FreeVec(GadgetText);
	if (FilteredBodyText) FreeVec(FilteredBodyText);

	return (Return_Value);

} /* Do_RequestChoice */


static
STRPTR ComposeGadgetText(struct localdata *ld,
                         STRPTR * Gadgets)
{
	STRPTR GadgetText, BufferPos;
	int    GadgetLength = 0;
	int    CurrentGadget;

	for (CurrentGadget = 0; Gadgets[CurrentGadget]; CurrentGadget++)
	{
		GadgetLength += strlen(Gadgets[CurrentGadget]) + 1;
	}

	GadgetText = AllocVec(GadgetLength, MEMF_ANY);
	if (!GadgetText)
	{
		/* Set by AllocVec() already - Piru  SetIoErr(ERROR_NO_FREE_STORE); */
		PrintFault(IoErr(), ERROR_HEADER);
		return NULL;
	}

	BufferPos = GadgetText;
	for (CurrentGadget = 0; Gadgets[CurrentGadget]; CurrentGadget++)
	{
		int LabelLength = strlen(Gadgets[CurrentGadget]);
		CopyMem(Gadgets[CurrentGadget], BufferPos, LabelLength);
		if (Gadgets[CurrentGadget + 1])
		{
			BufferPos[LabelLength] = '|';
			BufferPos += LabelLength + 1;
		}
		else
			BufferPos[LabelLength] = '\0';
	}

	return GadgetText;

} /* ComposeGadgetText */
